@@ -1,22 +1,10 @@
-# -*- coding: utf-8 -*-
-
-########################################################################
-#
-# License: BSD
-# Created: October 2, 2004
-# Author:  Ivan Vilata i Balaguer - reverse:net.selidor@ivan
-#
-# $Id$
-#
-########################################################################
-
 """A file interface to nodes for PyTables databases.
 
 The FileNode module provides a file interface for using inside of
 PyTables database files.  Use the new_node() function to create a brand
 new file node which can be read and written as any ordinary Python
 file.  Use the open_node() function to open an existing (i.e. created
-with new_node()) node for read-only or read-write access.  Read acces
+with new_node()) node for read-only or read-write access.  Read access
 is always available.  Write access (enabled on new files and files
 opened with mode 'a+') only allows appending data to a file node.
 
@@ -29,19 +17,18 @@ See :ref:`filenode_usersguide` for instructions on use.
    compliant with the interfaces defined in the :mod:`io` module.
 
 """
-from __future__ import absolute_import
 
 import io
 import os
 import re
 import warnings
+from pathlib import Path
 
 import numpy as np
 
-import tables
+import tables as tb
 
-
-NodeType = 'file'
+NodeType = "file"
 """Value for NODE_TYPE node system attribute."""
 
 NodeTypeVersions = [1, 2]
@@ -54,12 +41,12 @@ class RawPyTablesIO(io.RawIOBase):
     # A lambda to turn a size into a shape, for each version.
     _size_to_shape = [
         None,
-        lambda l: (l, 1),
-        lambda l: (l, ),
+        lambda n: (n, 1),
+        lambda m: (m,),
     ]
 
     def __init__(self, node, mode=None):
-        super(RawPyTablesIO, self).__init__()
+        super().__init__()
 
         self._check_node(node)
         self._check_attributes(node)
@@ -81,17 +68,15 @@ class RawPyTablesIO(io.RawIOBase):
     @property
     def mode(self):
         """File mode."""
-
         return self._mode
 
-    #def tell(self) -> int:
+    # def tell(self) -> int:
     def tell(self):
         """Return current stream position."""
-
-        self._checkClosed()
+        self._check_closed()
         return self._pos
 
-    #def seek(self, pos: int, whence: int = 0) -> int:
+    # def seek(self, pos: int, whence: int = 0) -> int:
     def seek(self, pos, whence=0):
         """Change stream position.
 
@@ -106,17 +91,16 @@ class RawPyTablesIO(io.RawIOBase):
         Return the new absolute position.
 
         """
-
-        self._checkClosed()
+        self._check_closed()
         try:
             pos = pos.__index__()
-        #except AttributeError as err:
-            #raise TypeError("an integer is required") from err
+        # except AttributeError as err:
+        #     raise TypeError("an integer is required") from err
         except AttributeError:
             raise TypeError("an integer is required")
         if whence == 0:
             if pos < 0:
-                raise ValueError("negative seek position %r" % (pos,))
+                raise ValueError(f"negative seek position {pos!r}")
             self._pos = pos
         elif whence == 1:
             self._pos = max(0, self._pos + pos)
@@ -126,7 +110,7 @@ class RawPyTablesIO(io.RawIOBase):
             raise ValueError("invalid whence value")
         return self._pos
 
-    #def seekable(self) -> bool:
+    # def seekable(self) -> bool:
     def seekable(self):
         """Return whether object supports random access.
 
@@ -134,35 +118,32 @@ class RawPyTablesIO(io.RawIOBase):
         method may need to do a test seek().
 
         """
-
         return True
 
-    #def fileno(self) -> int:
+    # def fileno(self) -> int:
     def fileno(self):
-        """Returns underlying file descriptor if one exists.
+        """Return underlying file descriptor if one exists.
 
         An IOError is raised if the IO object does not use a file
         descriptor.
 
         """
+        self._check_closed()
+        return self._node._v_file.fileno()
 
-        self._checkClosed()
-        self._node._v_file.fileno()
-
-    #def close(self) -> None:
+    # def close(self) -> None:
     def close(self):
         """Flush and close the IO object.
 
         This method has no effect if the file is already closed.
 
         """
-
         if not self.closed:
-            if getattr(self._node, '_v_file', None) is None:
+            if getattr(self._node, "_v_file", None) is None:
                 warnings.warn("host PyTables file is already closed!")
 
         try:
-            super(RawPyTablesIO, self).close()
+            super().close()
         finally:
             # Release node object to allow closing the file.
             self._node = None
@@ -173,11 +154,10 @@ class RawPyTablesIO(io.RawIOBase):
         This is not implemented for read-only and non-blocking streams.
 
         """
-
-        self._checkClosed()
+        self._check_closed()
         self._node.flush()
 
-    #def truncate(self, pos: int = None) -> int:
+    # def truncate(self, pos: int = None) -> int:
     def truncate(self, pos=None):
         """Truncate file to size bytes.
 
@@ -188,44 +168,41 @@ class RawPyTablesIO(io.RawIOBase):
         since data can not be rewritten nor deleted.
 
         """
-
-        self._checkClosed()
+        self._check_closed()
         self._checkWritable()
 
         if pos is None:
             pos = self._pos
         elif pos < 0:
-            raise ValueError("negative truncate position %r" % (pos,))
+            raise ValueError(f"negative truncate position {pos!r}")
 
         if pos < self._node.nrows:
-            raise IOError("truncating is only allowed for growing a file")
+            raise OSError("truncating is only allowed for growing a file")
         self._append_zeros(pos - self._node.nrows)
 
         return self.seek(pos)
 
-    #def readable(self) -> bool:
+    # def readable(self) -> bool:
     def readable(self):
         """Return whether object was opened for reading.
 
         If False, read() will raise IOError.
 
         """
-
         mode = self._mode
-        return 'r' in mode or '+' in mode
+        return "r" in mode or "+" in mode
 
-    #def writable(self) -> bool:
+    # def writable(self) -> bool:
     def writable(self):
         """Return whether object was opened for writing.
 
         If False, write() and truncate() will raise IOError.
 
         """
-
         mode = self._mode
-        return 'w' in mode or 'a' in mode or '+' in mode
+        return "w" in mode or "a" in mode or "+" in mode
 
-    #def readinto(self, b: bytearray) -> int:
+    # def readinto(self, b: bytearray) -> int:
     def readinto(self, b):
         """Read up to len(b) bytes into b.
 
@@ -233,8 +210,7 @@ class RawPyTablesIO(io.RawIOBase):
         is set not to block as has no data to read.
 
         """
-
-        self._checkClosed()
+        self._check_closed()
         self._checkReadable()
 
         if self._pos >= self._node.nrows:
@@ -245,10 +221,10 @@ class RawPyTablesIO(io.RawIOBase):
         stop = self._pos + n
 
         # XXX optimized path
-        #if stop <= self._node.nrows and isinstance(b, np.ndarray):
-        #    self._node.read(start, stop, out=b)
-        #    self._pos += n
-        #    return n
+        # if stop <= self._node.nrows and isinstance(b, np.ndarray):
+        #     self._node.read(start, stop, out=b)
+        #     self._pos += n
+        #     return n
 
         if stop > self._node.nrows:
             stop = self._node.nrows
@@ -261,9 +237,9 @@ class RawPyTablesIO(io.RawIOBase):
 
         return n
 
-    #def readline(self, limit: int = -1) -> bytes:
+    # def readline(self, limit: int = -1) -> bytes:
     def readline(self, limit=-1):
-        """Read and return a line from the stream.
+        r"""Read and return a line from the stream.
 
         If limit is specified, at most limit bytes will be read.
 
@@ -272,14 +248,13 @@ class RawPyTablesIO(io.RawIOBase):
         terminator(s) recognized.
 
         """
-
-        self._checkClosed()
+        self._check_closed()
         self._checkReadable()
 
         chunksize = self._node.chunkshape[0] if self._node.chunkshape else -1
 
         # XXX: check
-        lsep = b'\n'
+        lsep = b"\n"
         lseplen = len(lsep)
 
         # Set the remaining bytes to read to the specified size.
@@ -325,7 +300,7 @@ class RawPyTablesIO(io.RawIOBase):
             elif lseplen > 1 and (limit <= 0 or remsize > 0):
                 # Seek back a little since the end of the read string
                 # may have fallen in the middle of the line separator.
-                obuff = ibuff[:-lseplen + 1]
+                obuff = ibuff[: -lseplen + 1]
                 self.seek(-lseplen + 1, 1)
                 remsize += lseplen - 1
             else:  # eolindex<0 and (lseplen<=1 or (limit>0 and remsize<=0))
@@ -340,9 +315,9 @@ class RawPyTablesIO(io.RawIOBase):
             if limit > 0 and remsize <= 0:
                 finished = True
 
-        return b''.join(partial)
+        return b"".join(partial)
 
-    #def write(self, b: bytes) -> int:
+    # def write(self, b: bytes) -> int:
     def write(self, b):
         """Write the given buffer to the IO stream.
 
@@ -350,8 +325,7 @@ class RawPyTablesIO(io.RawIOBase):
         len(b).
 
         """
-
-        self._checkClosed()
+        self._check_closed()
         self._checkWritable()
 
         if isinstance(b, str):
@@ -371,29 +345,29 @@ class RawPyTablesIO(io.RawIOBase):
 
         # Append data.
         self._node.append(
-            np.ndarray(buffer=b, dtype=self._vtype, shape=self._vshape(n)))
+            np.ndarray(buffer=b, dtype=self._vtype, shape=self._vshape(n))
+        )
 
         self._pos += n
 
         return n
 
-    def _checkClosed(self):
-        """Checks if file node is open.
+    def _check_closed(self):
+        """Check if file node is open.
 
         Checks whether the file node is open or has been closed. In the
         second case, a ValueError is raised. If the host PyTables has
         been closed, ValueError is also raised.
 
         """
-
-        super(RawPyTablesIO, self)._checkClosed()
-        if getattr(self._node, '_v_file', None) is None:
+        # super()._check_closed()
+        if getattr(self._node, "_v_file", None) is None:
             raise ValueError("host PyTables file is already closed!")
 
     def _check_node(self, node):
-        if not isinstance(node, tables.EArray):
+        if not isinstance(node, tb.EArray):
             raise TypeError('the "node" parameter should be a tables.EArray')
-        if not isinstance(node.atom, tables.UInt8Atom):
+        if not isinstance(node.atom, tb.UInt8Atom):
             raise TypeError('only nodes with atom "UInt8Atom" are allowed')
 
     def _check_mode(self, mode):
@@ -407,7 +381,7 @@ class RawPyTablesIO(io.RawIOBase):
         reading = "r" in modes
         writing = "w" in modes
         appending = "a" in modes
-        #updating = "+" in modes
+        # updating = "+" in modes
         text = "t" in modes
         binary = "b" in modes
 
@@ -427,23 +401,26 @@ class RawPyTablesIO(io.RawIOBase):
 
     def _cross_check_mode(self, mode, h5filemode):
         # XXX: check
-        #readable = bool('r' in mode or '+' in mode)
-        #h5readable = bool('r' in h5filemode or '+' in h5filemode)
+        # readable = bool('r' in mode or '+' in mode)
+        # h5readable = bool('r' in h5filemode or '+' in h5filemode)
         #
-        #if readable and not h5readable:
-        #    raise ValueError("RawPyTablesIO can't be open in read mode if "
-        #                     "the underlying hdf5 file is not readable")
+        # if readable and not h5readable:
+        #     raise ValueError("RawPyTablesIO can't be open in read mode if "
+        #                      "the underlying hdf5 file is not readable")
 
-        writable = bool('w' in mode or 'a' in mode or '+' in mode)
-        h5writable = bool('w' in h5filemode or 'a' in h5filemode or
-                          '+' in h5filemode)
+        writable = bool("w" in mode or "a" in mode or "+" in mode)
+        h5writable = bool(
+            "w" in h5filemode or "a" in h5filemode or "+" in h5filemode
+        )
 
         if writable and not h5writable:
-            raise ValueError("RawPyTablesIO can't be open in write mode if "
-                             "the underlying hdf5 file is not writable")
+            raise ValueError(
+                "RawPyTablesIO can't be open in write mode if "
+                "the underlying hdf5 file is not writable"
+            )
 
     def _check_attributes(self, node):
-        """Checks file node-specific attributes.
+        """Check file node-specific attributes.
 
         Checks for the presence and validity
         of the system attributes 'NODE_TYPE' and 'NODE_TYPE_VERSION'
@@ -451,36 +428,35 @@ class RawPyTablesIO(io.RawIOBase):
         ValueError is raised if an attribute is missing or incorrect.
 
         """
-
         attrs = node.attrs
-        ltype = getattr(attrs, 'NODE_TYPE', None)
-        ltypever = getattr(attrs, 'NODE_TYPE_VERSION', None)
+        ltype = getattr(attrs, "NODE_TYPE", None)
+        ltypever = getattr(attrs, "NODE_TYPE_VERSION", None)
 
         if ltype != NodeType:
-            raise ValueError("invalid type of node object: %s" % (ltype,))
+            raise ValueError(f"invalid type of node object: {ltype}")
         if ltypever not in NodeTypeVersions:
             raise ValueError(
-                "unsupported type version of node object: %s" % (ltypever,))
-
+                f"unsupported type version of node object: {ltypever}"
+            )
 
     def _append_zeros(self, size):
-        """_append_zeros(size) -> None.  Appends a string of zeros.
+        """Append a string of zeros.
 
         Appends a string of 'size' zeros to the array,
         without moving the file pointer.
 
         """
-
         # Appending an empty array would raise an error.
         if size == 0:
             return
 
         # XXX This may be redone to avoid a potentially large in-memory array.
         self._node.append(
-            np.zeros(dtype=self._vtype, shape=self._vshape(size)))
+            np.zeros(dtype=self._vtype, shape=self._vshape(size))
+        )
 
 
-class FileNodeMixin(object):
+class FileNodeMixin:
     """Mixin class for FileNode objects.
 
     It provides access to the attribute set of the node that becomes
@@ -491,33 +467,25 @@ class FileNodeMixin(object):
     """
 
     # The attribute set property methods.
-    def _get_attrs(self):
-        """Returns the attribute set of the file node."""
-
-        #sefl._checkClosed()
+    @property
+    def attrs(self):
+        """Return the attribute set of the file node."""
+        # self._check_closed()
         return self._node.attrs
 
-
+    @attrs.setter
     def _set_attrs(self, value):
-        """set_attrs(string) -> None.  Raises ValueError."""
-
+        """Raise ValueError."""
         raise ValueError("changing the whole attribute set is not allowed")
 
-
+    @attrs.deleter
     def _del_attrs(self):
-        """del_attrs() -> None.  Raises ValueError."""
-
+        """Raise ValueError."""
         raise ValueError("deleting the whole attribute set is not allowed")
 
 
-    # The attribute set property.
-    attrs = property(
-        _get_attrs, _set_attrs, _del_attrs,
-        "A property pointing to the attribute set of the file node.")
-
-
 class ROFileNode(FileNodeMixin, RawPyTablesIO):
-    """Creates a new read-only file node.
+    r"""Creates a new read-only file node.
 
     Creates a new read-only file node associated with the specified
     PyTables node, providing a standard Python file interface to it.
@@ -547,16 +515,17 @@ class ROFileNode(FileNodeMixin, RawPyTablesIO):
     """
 
     def __init__(self, node):
-        RawPyTablesIO.__init__(self, node, 'r')
+        RawPyTablesIO.__init__(self, node, "r")
         self._checkReadable()
 
     @property
     def node(self):
+        """Node property."""
         return self._node
 
 
 class RAFileNode(FileNodeMixin, RawPyTablesIO):
-    """Creates a new read-write file node.
+    r"""Creates a new read-write file node.
 
     The first syntax opens the specified PyTables node, while the
     second one creates a new node in the specified PyTables file.
@@ -598,7 +567,12 @@ class RAFileNode(FileNodeMixin, RawPyTablesIO):
     ]
 
     __allowed_init_kwargs = [
-        'where', 'name', 'title', 'filters', 'expectedsize']
+        "where",
+        "name",
+        "title",
+        "filters",
+        "expectedsize",
+    ]
 
     def __init__(self, node, h5file, **kwargs):
         if node is not None:
@@ -611,53 +585,54 @@ class RAFileNode(FileNodeMixin, RawPyTablesIO):
             for kwarg in kwargs:
                 if kwarg not in self.__allowed_init_kwargs:
                     raise TypeError(
-                        "%s keyword argument is not allowed" % repr(kwarg))
+                        "%s keyword argument is not allowed" % repr(kwarg)
+                    )
 
             # Turn 'expectedsize' into 'expectedrows'.
-            if 'expectedsize' in kwargs:
+            if "expectedsize" in kwargs:
                 # These match since one byte is stored per row.
-                expectedrows = kwargs['expectedsize']
+                expectedrows = kwargs["expectedsize"]
                 kwargs = kwargs.copy()
-                del kwargs['expectedsize']
-                kwargs['expectedrows'] = expectedrows
+                del kwargs["expectedsize"]
+                kwargs["expectedrows"] = expectedrows
 
             # Create a new array in the specified PyTables file.
             self._version = NodeTypeVersions[-1]
             shape = self._byte_shape[self._version]
             node = h5file.create_earray(
-                atom=tables.UInt8Atom(), shape=shape, **kwargs)
+                atom=tb.UInt8Atom(), shape=shape, **kwargs
+            )
 
             # Set the node attributes, else remove the array itself.
             try:
                 self._set_attributes(node)
             except RuntimeError:
-                h5file.remove_node(kwargs['where'], kwargs['name'])
+                h5file.remove_node(kwargs["where"], kwargs["name"])
                 raise
 
-        RawPyTablesIO.__init__(self, node, 'a+')
+        RawPyTablesIO.__init__(self, node, "a+")
         self._checkReadable()
         self._checkWritable()
 
     @property
     def node(self):
+        """Node property."""
         return self._node
 
-    def _set_attributes(self, node):
-        """_set_attributes(node) -> None.  Adds file node-specific attributes.
+    def _set_attributes(self, node) -> None:
+        """Add file node-specific attributes.
 
         Sets the system attributes 'NODE_TYPE' and 'NODE_TYPE_VERSION'
         in the specified PyTables node (leaf).
 
         """
-
         attrs = node.attrs
         attrs.NODE_TYPE = NodeType
         attrs.NODE_TYPE_VERSION = NodeTypeVersions[-1]
 
 
-
 def new_node(h5file, **kwargs):
-    """Creates a new file node object in the specified PyTables file object.
+    """Create a new file node object in the specified PyTables file object.
 
     Additional named arguments where and name must be passed to specify where
     the file node is to be created. Other named arguments such as title and
@@ -667,14 +642,11 @@ def new_node(h5file, **kwargs):
     file size in bytes, may also be passed. It returns the file node object.
 
     """
-
     return RAFileNode(None, h5file, **kwargs)
 
 
-
-
-def open_node(node, mode='r'):
-    """Opens an existing file node.
+def open_node(node, mode="r"):
+    """Open an existing file node.
 
     Returns a file node object from the existing specified PyTables
     node. If mode is not specified or it is 'r', the file can only be
@@ -683,19 +655,17 @@ def open_node(node, mode='r'):
     positioned at the end of the file.
 
     """
-
-    if mode == 'r':
+    if mode == "r":
         return ROFileNode(node)
-    elif mode == 'a+':
+    elif mode == "a+":
         return RAFileNode(node, None)
     else:
-        raise IOError("invalid mode: %s" % (mode,))
+        raise OSError(f"invalid mode: {mode}")
 
 
-
-
-def save_to_filenode(h5file, filename, where, name=None, overwrite=False,
-                     title="", filters=None):
+def save_to_filenode(
+    h5file, filename, where, name=None, overwrite=False, title="", filters=None
+):
     """Save a file's contents to a filenode inside a PyTables file.
 
     .. versionadded:: 3.2
@@ -734,15 +704,17 @@ def save_to_filenode(h5file, filename, where, name=None, overwrite=False,
        during the life of this object.
 
     """
+    path = Path(filename).resolve()
+
     # sanity checks
-    if not os.access(filename, os.R_OK):
-        raise IOError("The file '%s' could not be read" % filename)
-    if isinstance(h5file, tables.file.File) and h5file.mode == "r":
-        raise IOError("The file '%s' is opened read-only" % h5file.filename)
+    if not os.access(path, os.R_OK):
+        raise OSError(f"The file '{path}' could not be read")
+    if isinstance(h5file, tb.file.File) and h5file.mode == "r":
+        raise OSError(f"The file '{h5file.filename}' is opened read-only")
 
     # guess filenode's name if necessary
     if name is None:
-        if isinstance(where, tables.group.Group):
+        if isinstance(where, tb.group.Group):
             name = os.path.split(filename)[1]
         if isinstance(where, str):
             if where.endswith("/"):
@@ -753,38 +725,39 @@ def save_to_filenode(h5file, filename, where, name=None, overwrite=False,
                 name = nodepath[-1]
 
     # sanitize name if necessary
-    if not tables.path._python_id_re.match(name):
-        name = re.sub('(?![a-zA-Z0-9_]).', "_",
-                      re.sub('^(?![a-zA-Z_]).', "_", name))
+    if not tb.path._python_id_re.match(name):
+        name = re.sub(
+            "(?![a-zA-Z0-9_]).", "_", re.sub("^(?![a-zA-Z_]).", "_", name)
+        )
 
-    new_h5file = not isinstance(h5file, tables.file.File)
-    f = tables.File(h5file, "a") if new_h5file else h5file
+    new_h5file = not isinstance(h5file, tb.file.File)
+    f = tb.File(h5file, "a") if new_h5file else h5file
 
     # check for already existing filenode
     try:
-        n = f.get_node(where=where, name=name)
+        f.get_node(where=where, name=name)
         if not overwrite:
             if new_h5file:
                 f.close()
-            raise IOError("Specified node already exists in file '%s'" %
-                          f.filename)
-    except tables.NoSuchNodeError:
+            raise OSError(
+                f"Specified node already exists in file '{f.filename}'"
+            )
+    except tb.NoSuchNodeError:
         pass
 
     # read data from disk
-    with open(filename, "rb") as fd:
-        data = fd.read()
+    data = path.read_bytes()
 
     # remove existing filenode if present
     try:
         f.remove_node(where=where, name=name)
-    except tables.NoSuchNodeError:
+    except tb.NoSuchNodeError:
         pass
 
     # write file's contents to filenode
     fnode = new_node(f, where=where, name=name, title=title, filters=filters)
     fnode.write(data)
-    fnode.attrs._filename = os.path.split(filename)[1]
+    fnode.attrs._filename = path.name
     fnode.close()
 
     # cleanup
@@ -792,8 +765,9 @@ def save_to_filenode(h5file, filename, where, name=None, overwrite=False,
         f.close()
 
 
-def read_from_filenode(h5file, filename, where, name=None, overwrite=False,
-                       create_target=False):
+def read_from_filenode(
+    h5file, filename, where, name=None, overwrite=False, create_target=False
+):
     r"""Read a filenode from a PyTables file and write its contents to a file.
 
     .. versionadded:: 3.2
@@ -825,11 +799,13 @@ def read_from_filenode(h5file, filename, where, name=None, overwrite=False,
       given target ``filename`` will be created.
 
     """
-    new_h5file = not isinstance(h5file, tables.file.File)
-    f = tables.File(h5file, "r") if new_h5file else h5file
+    path = Path(filename).resolve()
+
+    new_h5file = not isinstance(h5file, tb.file.File)
+    f = tb.File(h5file, "r") if new_h5file else h5file
     try:
         fnode = open_node(f.get_node(where=where, name=name))
-    except tables.NoSuchNodeError:
+    except tb.NoSuchNodeError:
         fnode = None
         for n in f.walk_nodes(where=where, classname="EArray"):
             if n.attrs._filename == name:
@@ -837,46 +813,40 @@ def read_from_filenode(h5file, filename, where, name=None, overwrite=False,
                 break
         if fnode is None:
             f.close()
-            raise tables.NoSuchNodeError("A filenode '%s' cannot be found at "
-                                         "'%s'" % (name, where))
+            raise tb.NoSuchNodeError(
+                "A filenode '%s' cannot be found at " "'%s'" % (name, where)
+            )
 
     # guess output filename if necessary
-    if os.path.isdir(filename) or filename.endswith(os.path.sep):
+    # TODO: pathlib.Path strips trailing slash automatically :-(
+    if path.is_dir() or filename.endswith(os.path.sep):
         try:
-            filename = os.path.join(filename, fnode.node.attrs._filename)
+            path = path / fnode.node.attrs._filename
         except Exception:
-            filename = os.path.join(filename, fnode.node.name)
+            path = path / fnode.node.name
 
-    if os.access(filename, os.R_OK) and not overwrite:
+    if os.access(path, os.R_OK) and not overwrite:
         if new_h5file:
             f.close()
-        raise IOError("The file '%s' already exists" % filename)
+        raise OSError(f"The file '{path}' already exists")
 
     # create folder hierarchy if necessary
-    if create_target and not os.path.isdir(os.path.split(filename)[0]):
-        os.makedirs(os.path.split(filename)[0])
+    if create_target:
+        path.parent.mkdir(parents=True, exist_ok=True)
 
-    if not os.access(os.path.split(filename)[0], os.W_OK):
+    if not os.access(path.parent, os.W_OK):
         if new_h5file:
             f.close()
-        raise IOError("The file '%s' cannot be written to" % filename)
+        raise OSError("The file '%s' cannot be written to" % filename)
 
     # read data from filenode
     data = fnode.read()
     fnode.close()
 
     # store data to file
-    with open(filename, "wb") as fd:
-        fd.write(data)
+    path.write_bytes(data)
 
     # cleanup
     del data
     if new_h5file:
         f.close()
-
-
-## Local Variables:
-## mode: python
-## py-indent-offset: 4
-## tab-width: 4
-## End:
